/*------------------------------------------------------------------------
 Alternating direction augmented Lagrangian method 

 Common routines
  
 Auther: Zaiwen Wen
 Date:   08-01-2009 
------------------------------------------------------------------------*/

#include <math.h>
#include "string.h"
#include "blas.h"

/*#ifndef intISINT
 This line is needed for versions of MATLAB prior to 7.3.
typedef int int;
typedef int mwSize;
#endif
 */

#if !defined(MAX)
#define	MAX(A, B)	((A) > (B) ? (A) : (B))
#endif

#if !defined(MIN)
#define	MIN(A, B)	((A) < (B) ? (A) : (B))
#endif

/*-----------------------------------------------------------------------
 * X is dense matrix, XX is dense vector
 * C is a sparse matrix, CX is a column of C
 *
 * Sparse matrix vector operation:
 * X*CX
 * CX'*XX
/*-----------------------------------------------------------------------
 
/*
Sparse matrices have a different storage convention from that of full matrices
in MATLAB. The parameters pr and pi are still arrays of double-precision numbers,
but these arrays contain only nonzero data elements. There are three additional
parameters: nzmax, ir, and jc.
 
nzmax is an integer that contains the length of ir, pr, and, if it exists, pi.
It is the maximum possible number of nonzero elements in the sparse matrix.
 
ir points to an integer array of length nzmax containing the row indices of
the corresponding elements in pr and pi.
 
jc points to an integer array of length n+1, where n is the number of columns
in the sparse matrix. The jc array contains column index information. If the
jth column of the sparse matrix has any nonzero elements, jc[j] is the index
in ir and pr (and pi if it exists) of the first nonzero element in the jth column,
and jc[j+1] - 1 is the index of the last nonzero element in that column.
For the jth column of the sparse matrix, jc[j] is the total number of nonzero
elements in all preceding columns. The last element of the jc array, jc[n],
is equal to nnz, the number of nonzero elements in the entire sparse matrix.
If nnz is less than nzmax, more nonzero entries can be inserted into the array
without allocating additional storage.
 */

/* print a matrix */
void printVec(int m,  double *X);

/* print a matrix */
void printMat(int m, int n,  double *X);

void printMatInt(int m, int n,  int *X);

/* sign of a variable */
double sign(double a);


/* create a zero vector */
void zeros(int n,  double *X);


/* create a zero vector */
void ones(int n,  double *X);


/* summation of a symmetric matrix */
double sumSYMX(int n,  double *X);


/* copy W to Wold*/
void copySYM(int n, double *W, double *Wold);


/* scale W = alpha*W*/
void scalSYM(int n, double *W, double alpha);

/* F-norm of (W-Wold) */
double nrmdiffSYM(int n, double *Wold, double *W);

/* F-norm of W */
double nrmSYM(int n, double *W);

/* if plus = 0, X = Y-Z
    if plus = 1, X = Y+Z
 */
void diffSYM(int n, double *X, double *Y, double *Z, int plus);

/*  X = a*Y+b*Z  */
void addSYM(int n, double *X, double a, double *Y, double b, double *Z);

double ddotSYM(int n,  double *X, double *Z);

double matrix_norm(int n, double *a, int type);


void AltSDPPROJFEIG(int n, int nsq, double mu, double *S, double *X, double rho, double *XX, int* compEigS, double *W, double *WW, 
             double *D, int* nev, double *work, int *iwork);

/* eigen value decomposition */
void eigdsyevx(int n, double *W, double VL, double VU, double abstol,
        double *V, double *D, int *nev, int lwork, double *work, int *iwork, int *ifail);



/* projection by  using  dsyevx in LAPACK */
void AltSDPPROJ(int n, int nsq, double mu, double *S, double *X, double rho, double*XX, int* compEigS, double *W, double *WW, 
            double VL, double VU, double abstol, double *V, double *D, int* nev, int lwork, double *work, int *iwork, int *ifail );


void lbfgs_(int* n, int* m, double* x,
	double* f, double* g, int* diagco, double*  diag,
	int* iprint, double* eps, double* xtol, double* wa,
	int* iflag);

void AltSDPPROJLBFGS(int n,  double mu, double *S, double *X,  double rho, double*XX, int* compEigS, double *W, double *WW,
              double *V, int* nev,  int mstore, double eps, double xtol, int mxitr, int record, double *workLB, double *work);




