/***********************************************************************
* MexDSYEVX.c : C mex file for the routine DSYEVX in LAPACK
*
* mex -O -lmwlapack -lmwblas MexDSYEVX.c
*
*   opts.jobz= 1; opts.range = 2; opts.abstol = 1e-8;  
*  [V,D,nv] = MexDSYEVX(W, vl, vu, opts);
*
* opt.jobz = 1 (default), compute both eigenvectors and eigenvalues;
*         = 2, compute only eigenvalues.
*
*
* opt.range = 1 (default), compute all;
*         = 2, compute  eigenvalues between [vl, vu].
*         = 3, compute  vl-th to vu-th eigenvalues.
*
*opts.abstol   tolerance
*
* Zaiwen Wen
* August 2009
***********************************************************************/

#include <math.h>
#include <mex.h>
#include <matrix.h>
#include <string.h> /* needed for memcpy() */

#if !defined(MX_API_VER) || ( MX_API_VER < 0x07030000 )
typedef int mwIndex;
typedef int mwSize;
#endif

/* print a matrix */
void printVec(mwIndex m,  double *X)
{
    mwIndex di;
    mexPrintf("\n");
    for(di=0; di<m; di++)
            mexPrintf("%3.2e \t", X[di]);
    mexPrintf("\n");
}

/* print a matrix */
void printMat(mwIndex m, mwIndex n,  double *X)
{
    mwIndex di, dj;
    mexPrintf("\n");
    for(di=0; di<m; di++)
    {
        for(dj=0; dj<n; dj++)
            mexPrintf("%3.2e \t", *(X+dj*m+di));
        mexPrintf("\n");
    }
}

/**********************************************************
* 
***********************************************************/
void mexFunction(
      int nlhs,   mxArray  *plhs[], 
      int nrhs,   const mxArray  *prhs[] )

{    double   *A, *AA, *nV, *V, *D, VL, VU, abstol, *work, *work2, *tmp;  

     mwIndex  subs[2];
     mwSize   nsubs=2;
     mwSize   m, n, IL, IU, lwork, *iwork, *ifail, info, options; 
     char     *jobz="V";
     char     *uplo="U"; 
     char     *range="A";

/* CHECK FOR PROPER NUMBER OF ARGUMENTS */

   if (nrhs > 4){
      mexErrMsgTxt("mexeig: requires at most 4 input arguments."); }
   if (nlhs > 3){ 
      mexErrMsgTxt("mexeig: requires at most 3 output argument."); }   

/* CHECK THE DIMENSIONS */

    m = mxGetM(prhs[0]); 
    n = mxGetN(prhs[0]); 
    if (m != n) { 
       mexErrMsgTxt("mexeig: matrix must be square."); }
    if (mxIsSparse(prhs[0])) {
       mexErrMsgTxt("mexeig: sparse matrix not allowed."); }   
    A = mxGetPr(prhs[0]);     

    options = 1;    abstol = 1e-6;
    /*Get fields from opts */
    tmp = mxGetPr(mxGetField(prhs[3], 0, "abstol"));  abstol  = *tmp;
    tmp = mxGetPr(mxGetField(prhs[3], 0, "jobz"));    options  = (int)*tmp;
    if (options==1) { jobz="V"; } else if (options==2) { jobz="N"; } 
    tmp = mxGetPr(mxGetField(prhs[3], 0, "range"));   options = (int)*tmp;
    if (options==1) 
    { 
        range="A"; 
    } 
    else if (options==2) 
    { 
        range="V"; VL = *mxGetPr(prhs[1]);   VU = *mxGetPr(prhs[2]);  
    }
    else if (options == 3)
    { 
        range="I"; IL = (int)*mxGetPr(prhs[1]); IU = (int)*mxGetPr(prhs[2]);
    }   

 


    /***** create return argument *****/
    plhs[0] = mxCreateDoubleMatrix(n,n,mxREAL); 
    V = mxGetPr(plhs[0]);  
    plhs[1] = mxCreateDoubleMatrix(n,1,mxREAL);
    D = mxGetPr(plhs[1]);  
    plhs[2] = mxCreateDoubleMatrix(1,1,mxREAL); 
    nV = mxGetPr(plhs[2]);

    /***** Do the computations in a subroutine *****/
    lwork  = 8*n;  
    work   =  mxCalloc(lwork,sizeof(double)); 
    iwork  =  mxCalloc(5*n, sizeof(mwSize));
    ifail  =  mxCalloc(n, sizeof(mwSize));  
    
    AA   =  mxCalloc(n*n,sizeof(double)); 

    memcpy(AA,mxGetPr(prhs[0]),(n*n)*sizeof(double));
    
    /*
    printMat(n,n,A);
    mexPrintf("jobz: %s, range: %s, tol: %3.2e \n", jobz, range, abstol);
    mexPrintf("VL: %3.2e, VU: %3.2e, IL: %d, IU: %d \n", VL, VU, IL, IU);   
     */
    
    dsyevx_(jobz, range, uplo, &n, AA, &n, &VL, &VU, &IL, &IU,
                        &abstol, &m, D, V, &n, work, &lwork, iwork,
                        ifail, &info);
    *nV = m;    
    
    /* mexPrintf("nV: %d\n",(int)*nV); */
    return;
 }
/**********************************************************/
